<?php
namespace PrestaShop\Module\LCVPrestaConnector;

/**
 * Gestionnaire de catégories
 * 
 */
class CategoriesManager
{
    /**
     * Synchroniseur
     * 
     * @var Syncer
     */
    private Syncer $syncer;

    /**
     * Catégories dynamiques gérées
     */
    public array $dynCats;

    /**
     * Mapping des catégories dynamiques
     */
    private array $mapping;

    /**
     * Liste exhaustive des catégories Prestashop de destination de mappage
     */
    private array $mappedCats;

    /**
     * Catégories des produits : pour chaque id_product, un tableau de catégories
     * 
     * @var array<int, array<int></int>
     */
    private array $productsCategories;

    /** 
     * Nouvelle instance
     * 
     * @param Syncer $syncer Synchroniseur
     */
    public function __construct(Syncer $syncer, ?array $productsId = null)
    {
        $this->syncer = $syncer;
        // Chargement des mappings, pour chaque catégorie dynamique
        $mapping = [];
        $cfg = $this->syncer->module->getCfg();

        $this->dynCats = $cfg->get(SyncConfiguration::CFG_DYN_CAT);

        foreach ($this->dynCats as $cat => $name)
        {
            $mapping[$cat] = $cfg->get(SyncConfiguration::CFG_MAP_DYN_CAT . $cat);
            if (!isset($mapping[$cat]))
                $mapping[$cat] = [];
        }

        $this->mapping = $mapping;
        $this->mappedCats = [];
        foreach ($mapping as $cat => $map)
        {
            foreach ($map as $code => $id_category)
            {
                if (is_array($id_category))
                {
                    foreach ($id_category as $id)
                        $this->mappedCats[$id] = true;
                }
                else
                    $this->mappedCats[$id_category] = true;
            }
        }

        // Et maintenant, chargement des catégories du produit depuis la base de données
        $this->productsCategories = [];
        $db = \Db::getInstance();
        $sql = 'SELECT id_product, id_category FROM ' . _DB_PREFIX_ . 'category_product';
        if ($productsId)
            $sql .= ' WHERE id_product IN (' . implode(',', $productsId) . ')';
        $res = $db->executeS($sql);
        if ($res === FALSE)
            throw new \Exception('Erreur lors de la récupération des catégories des produits : '.$db->getMsgError());
        foreach ($res as $row)
        {
            $id_product = (int)$row['id_product'];
            $id_category = (int)$row['id_category'];
            if (!isset($this->productsCategories[$id_product]))
                $this->productsCategories[$id_product] = [];
            $this->productsCategories[$id_product][] = $id_category;
        }
    }

    /*
     * Détermine si l'id de la catégorie renseignée est le résultat
     * probable d'un mappage
     */
    public function isMappedCategory(int $id_category) : bool
    {
        return isset($this->mappedCats[$id_category]) && $this->mappedCats[$id_category];
    }

    /**
     * Traduit un code de catégorie en ID de catégories Prestashop selon le mapping prévu
     * 
     * @param string $category Catégorie dynamique
     * @param string $code Code
     * @return array<int>ID de catégories
     */
    public function translateCode($category, $code) : array
    {
        if (isset($this->mapping[$category]))
        {
            $id_category = 0;
        
            if (isset($this->mapping[$category][$code]))
                $id_category = $this->mapping[$category][$code];

            if (!$id_category && isset($this->mapping[$category]["*"])) // catch all ?
                $id_category = $this->mapping[$category]["*"];

            if (is_array($id_category))
            {
                if (count($id_category) > 0 && (count($id_category) > 1 || $id_category[0] != 0))
                    return $id_category;
            }
            else if ($id_category)
                return [$id_category];
        }

        return [];
    }

    /**
     * Obtient le cache des catégories d'un produit
     */
    public function getProductCategories(int $id_product) : array
    {
        return isset($this->productsCategories[$id_product]) ? $this->productsCategories[$id_product] : [];
    }

    /**
     * Contrôle que les catégories passées en paramètre sont les mêmes que celles du produit
     * 
     * @param int $id_product ID du produit
     * @param array<int> $cats Catégories à vérifier
     * @param array<int> $productCats Catégories du produit (optionnel, autodétectées si non renseignées)
     * @return bool Vrai si les catégories sont les mêmes     
     */
    public function isCatsTheSame(int $id_product, array $cats, ?array $productCats = null) : bool
    {
        // Catégories actuelles du produits
        if (!isset($productCats))
            $productCats = $this->getProductCategories($id_product);

        // On compare que les tableaux sont équivalents (quelque soit l'ordre)        
        if (count($productCats) != count($cats))
            return false;

        foreach ($cats as $cat)
        {
            if (!in_array($cat, $productCats))
                return false;
        }

        foreach ($productCats as $cat)
        {
            if (!in_array($cat, $cats))
                return false;
        }

        return true;    // Ca a l'air ok !
    }

    /**
     * Enregistre un nouveau set de catégorie pour un produit
     * 
     * @param int $id_product ID du produit
     * @param array<int> $cats Catégories     
     */
    public function registerCategories(int $id_product, array $cats) : void
    {
        // Affectation
        $this->productsCategories[$id_product] = $cats;
    }
}